@@ -1,20 +1,21 @@
 #!/usr/bin/env python3
 
 import argparse
+import asyncio
 import io
 import json
 import logging
 import os
 import sys
 import copy
-import threading
-import time
 
-from typing import Dict, List, Optional
+from asyncio import AbstractEventLoop
+from datetime import datetime
 from joblib import Parallel, delayed, parallel_backend
 from jsonschema import Draft7Validator, FormatChecker
 from singer import get_logger
-from datetime import datetime, timedelta
+from threading import Thread
+from typing import Dict, List, Optional
 
 import pkg_resources
 from c8connector import C8Connector, Sample, ConfigAttributeType, Schema
@@ -29,7 +30,6 @@ from macrometa_target_snowflake.db_sync import DbSync
 from macrometa_target_snowflake.file_format import FileFormatTypes
 from macrometa_target_snowflake.exceptions import (
     RecordValidationException,
-    UnexpectedValueTypeException,
     InvalidValidationOperationException
 )
 
@@ -88,12 +88,6 @@ def get_snowflake_statics(config):
     return table_cache, file_format_type
 
 
-def flush_timer(batch_wait_limit_seconds, flush_event):
-    while True:
-        time.sleep(batch_wait_limit_seconds)
-        flush_event.set()
-
-
 # pylint: disable=too-many-locals,too-many-branches,too-many-statements,invalid-name
 def persist_lines(config, lines, table_cache=None, file_format_type: FileFormatTypes = None) -> None:
     """Main loop to read and consume singer messages from stdin
@@ -122,15 +116,15 @@ def persist_lines(config, lines, table_cache=None, file_format_type: FileFormatT
     stream_to_sync = {}
     total_row_count = {}
     batch_size_rows = config.get('batch_size_rows', DEFAULT_BATCH_SIZE_ROWS)
-    batch_wait_limit_seconds = config.get(
-        'batch_wait_limit_seconds', DEFAULT_BATCH_WAIT_TIME)
     archive_load_files = config.get('archive_load_files', False)
     archive_load_files_data = {}
 
-    flush_event = threading.Event()
-    flush_thread = threading.Thread(target=flush_timer, args=(batch_wait_limit_seconds, flush_event))
-    flush_thread.daemon = True
-    flush_thread.start()
+    time_schedule = {
+        'batch_wait_limit_seconds': config.get('batch_wait_limit_seconds', DEFAULT_BATCH_WAIT_TIME),
+        'last_executed_time': datetime.now(),
+    }
+    event_loop = setup_flush_task(time_schedule, records_to_load, row_count, stream_to_sync, config,
+                                  state, flushed_state)
 
     # Loop over lines from stdin
     for line in lines:
@@ -208,17 +202,10 @@ def persist_lines(config, lines, table_cache=None, file_format_type: FileFormatT
                     if max_value is None or max_value < incremental_key_value:
                         stream_archive_load_files_values['max'] = incremental_key_value
 
-            flush = False
             if row_count[stream] >= batch_size_rows:
-                flush = True
                 LOGGER.info("Flush triggered by batch_size_rows (%s) reached in %s",
                             batch_size_rows, stream)
-            elif flush_event.is_set():
-                flush = True
-                LOGGER.info("Flush triggered by batch_wait_limit_seconds (%s)",
-                            batch_wait_limit_seconds)
 
-            if flush:
                 # flush all streams, delete records if needed, reset counts and then emit current state
                 if config.get('flush_all_streams'):
                     filter_streams = None
@@ -236,11 +223,9 @@ def persist_lines(config, lines, table_cache=None, file_format_type: FileFormatT
                     archive_load_files_data,
                     filter_streams=filter_streams)
 
-                # Reset the flush_event
-                flush_event.clear()
-
                 # emit last encountered state
                 emit_state(copy.deepcopy(flushed_state))
+                time_schedule['last_executed_time'] = datetime.now()
 
         elif t == 'SCHEMA':
             if 'stream' not in o:
@@ -357,9 +342,11 @@ def persist_lines(config, lines, table_cache=None, file_format_type: FileFormatT
         # flush all streams one last time, delete records if needed, reset counts and then emit current state
         flushed_state = flush_streams(records_to_load, row_count, stream_to_sync, config, state, flushed_state,
                                       archive_load_files_data)
+        time_schedule['last_executed_time'] = datetime.now()
 
     # emit latest state
     emit_state(copy.deepcopy(flushed_state))
+    event_loop.stop()
 
 
 # pylint: disable=too-many-arguments
@@ -537,6 +524,42 @@ def flush_records(stream: str,
     db_sync.delete_from_stage(stream, s3_key)
 
 
+def setup_flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state,
+                     filter_streams=None) -> AbstractEventLoop:
+    event_loop = asyncio.new_event_loop()
+    Thread(target=start_background_loop, args=(
+        event_loop,), daemon=True).start()
+    asyncio.run_coroutine_threadsafe(
+        flush_task(time_schedule, streams, row_count, stream_to_sync,
+                   config, state, flushed_state, filter_streams),
+        event_loop)
+    return event_loop
+
+
+async def flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state,
+                     filter_streams=None) -> None:
+    while True:
+        timedelta = datetime.now() - time_schedule['last_executed_time']
+        if (
+            timedelta.total_seconds(
+            ) >= time_schedule['batch_wait_limit_seconds']
+            and sum(row_count.values()) > 0
+        ):
+            # flush all streams one last time, delete records if needed, reset counts and then emit current state.
+            flushed_state = flush_streams(streams, row_count, stream_to_sync, config, state, flushed_state,
+                                          filter_streams)
+            # emit latest state
+            emit_state(copy.deepcopy(flushed_state))
+            time_schedule['last_executed_time'] = datetime.now()
+        # Add sleep statement to ensure periodic execution
+        await asyncio.sleep(time_schedule['batch_wait_limit_seconds'])
+
+
+def start_background_loop(loop: asyncio.AbstractEventLoop) -> None:
+    asyncio.set_event_loop(loop)
+    loop.run_forever()
+
+
 class SnowflakeTargetConnector(C8Connector):
     """SnowflakeTargetConnector's C8Connector impl."""
 
